from tests.notes_xml import TestNotesXML
from tests.otp_versions_table import TestOTPVersionsTable
from tests.changes import TestChanges

__all__ = ["TestNotesXML", "TestOTPVersionsTable", "TestChanges"]
